//***************************************************************
//***
//***   File:     VECTORH.H++
//***   Function: Declarations of VectorH and derived classes
//***
//***   Compilation: Ok.
//***
//***   11-Dic-95      Carlos      Creation.
//***   20-May-96      Jorge       Added methods in Vector3D.
//***   26-Nov-96      Carlos      Added methods in Vector3D.
//***   21-Jan-97      Jorge       Added method in Vector3D.
//***   
//***************************************************************

#ifndef  VECTORH_HPP
#define  VECTORH_HPP

#include <Global.h++>
#include <Object.h++>

//*************************************************************

class VectorH :

// Vectors in homgeneous coordinates, modelling 
// points, vectors and normal vectors in 3D space
// The behavour is defined in derived classes.
//
/
{
  protected :
    
  Scalar c[4]  ;
        
         // array of values at each component of the vector
     
  VectorH( Scalar cx, Scalar cy, Scalar cz, Scalar cw ) ;
        
         // builds a VectorH, explicitly giving its components
         
  VectorH & operator *= ( const Matrix4x4 &m ) ;
  
         // aplies 'm' to the receiver and returns a
         // reference to the modified receiver 
             
  friend ostream & operator << ( ostream & os, const VectorH & v ) ;
       
         // dumps the vector 'v' on the output stream 'os'
         // returns a reference to 'os' after writing
         
  public:
         
  Scalar operator[]( const Axis axis ) const ;
  
         // returns the component of the receiver in the axis 'axis'
         
  VectorH & operator *= ( const Transform &t ) ;
  
         // applies 't' to the receiver and returns a
         // reference to the modified receiver
         
  VectorH operator * ( const Transform &t ) const ;
  
         // applies 't' to the reciver, and returns a
         // copy of the result.  

  Boolean IsValid() const ;

         // check if any of the components of the receiver
         // is an invalid floating value, and in this
         // case returns false. In any other case returns
         // true.   
  
} ;

//**************************************************************

class Vector3D : public VectorH

// Free Vectors in 3D, a free vector is defined as
// the diference vector from a static point to another one.
// The w component is always 0 for Vector3D instances. This 
// implies applying a traslation transformation does not
// modifies a instance.
//
//  Date       Author   Action
//  -------------------------------------------------
//  11-Dic-95  Carlos   Creation
//  20-May-96  Jorge    Added methods Module and Normalized
//  26-Nov-96  Carlos   Added 'ReflectedOn' and 'operator=='
//  21-Jan-97  Jorge    Added 'RefractedOn' 
//  27-Jan-97  Carlos   Added 'Vector3D::Vector3D( Axis axis )' 

{
  public:

  Vector3D() : VectorH(0.0,1.0,0.0,0.0) {}

         // default constructor

  Vector3D( Axis axis ) ;

         // creates the unit lenght vector parallel to the given axis
  
  Vector3D( Scalar cx, Scalar cy, Scalar cz ) ;
    
         // constructor, explicitly giving every component. 
         // the W component is set to cero.
        
  Vector3D & operator += ( const Vector3D &v ) ;
    
         // adds 'v' to the receiver, returns a reference to the 
         // modified receiver (impl. of  v += v )

  Vector3D operator + ( const Vector3D &v ) const ;
    
         // adds the receiver and 'v', then returns a copy of the 
         // result (impl. of v= v+v )
         
  Vector3D & operator -= ( const Vector3D &v ) ;
  
         // subtracts 'v' to the receiver, return a reference to the 
         // modified receiver (impl of v -= v ) 

  Vector3D operator - ( const Vector3D &v ) const ;
  
         // substract 'v' from the receiver, then returns a 
         // copy of the result (impl. of v= v+v )

  Scalar operator | ( const Vector3D &v ) const ;
  
         // performs inner product between the receiver and 'v'
         // then returns the result (impl of t= v|v )
         
  Vector3D & operator *= ( Scalar t ) ;
  
         // performs external product of the receiver and 't'
         // then returns a reference to the modified receiver
         
  Vector3D operator * ( Scalar t ) const  ;
  
        // performs external product between the receiver ant 't'
        // then returns a copy of the result ( impl. of  v= v*t )
  
  friend Vector3D operator * ( Scalar t, const Vector3D &v ) ;
  
        // performs external product between 't' and 'v', then
        // returns a copy of the result
        
  Vector3D operator * ( const Vector3D &v ) const ;
  
        // performs cross product of the receiver and 'v'
        // then returns a copy of the result
        
  Vector3D & operator *= ( const Transform &t )  ;
  
        // applies 't' to the receiver, and returns a 
        // reference to the modified receiver
        
  Vector3D operator * ( const Transform &t ) const ;
  
        // applies 't' to the receiver, and returns
        // a copy of the result
        
  Scalar Module ( ) const ;
    
         // It returns the vector module.

  Vector3D Normalized ( ) const ;
  
         // It returns a normalized copy of the reciever

  Vector3D ReflectedOn( const Vector3D &normal ) const ;

         // returns a vector in the same plane of the receiver
         // and 'normal', with an angle with the 'normal' equal
         // to that of the receiver and the 'normal', with oposite
         // sign.

  Vector3D RefractedOn ( const Vector3D &normal, Scalar irf ) const ;

         // returns a refrected vector with refraction angle irf.

  Boolean operator==( const Vector3D &v ) const ;

         // compares the receiver and 'v' returns 'true' if
         // they're equal, 'false' if not. This comparision
         // is directly based on Scalar comparision.
	 
  friend Vector3D RandPointOnSphere ( ) ;
  
         // It returns a Vector3D from random 3D point on Sphere
	 // with radius 1.
	
  friend Vector3D RandVectorCosineZ ( ) ;

         // returns a random vector with unit lenght and z>0
         // (on the positive hemisphere). The probability of
         // selecting the vector (x,y,z) is directly proportional
         // to the value 'z' 
         //
         //  note that : z == inner product of (x,y,z) and (0,0,1)
         //  that is   : z == cosine of the angle between (x,y,z) and Z axis

  friend Vector3D RandVectorPhongZ( Scalar n ) ;

         // returns a random vector with unit lenght and z>0
         // (on the positive hemisphere). The probability of
         // selecting the vector V is directly proportional
         // to the value  (V|Z)^n , where Z=(0,0,1), "|" 
         // means inner product and "^" means exponentiation
       
   
} ;

//**************************************************************

class Point3D : public VectorH

//  points in three dimensional space.
//  The w component of the instances should always be one,
//  except just after aplying a projection transformation, and before
//  normalization takes place. In this state, points should not
//  be applied any other transformation.
//
//  Date       Author   Action
//  -------------------------------------------------
//  11-Dic-95  Carlos   Creation

{
  public:

  Point3D() : VectorH(0.0,0.0,0.0,1.0) {}

         // default constructor

  Point3D( Scalar cx, Scalar cy, Scalar cz ) ;
    
        // constructor, explicitly giving every component. 
        // the W component is set to 1.
        
  Vector3D operator - ( const Point3D & p ) const ;
  
        // returns a copy of the difference vector
        // from 'p' to the receiver
    
  Point3D & operator += ( const Vector3D &v ) ;
  
        // adds 'v' to the receiver, then returns a reference to
        // the modified receiver (impl. of p += v )
        
  Point3D operator + ( const Vector3D &v ) const ;
  
        // adds the receiver and 'v', then returns a copy
        // of the result (impl. of  p = p + v )
        
  Point3D & operator -= ( const Vector3D &v ) ;
  
        // substracts 'v' from the receiver, then returns a reference to
        // the modified receiver (impl. of p -= v )
        
  Point3D operator - ( const Vector3D &v ) const ;
  
        // substract 'v' from the receiver, then returns a copy
        // of the result (impl. of  p = p - v )
      
  Point3D  & operator *= ( const Transform &t ) ;
  
        // applies 't' to the receiver, then returns a
        // reference to the modified receiver
        
  Point3D operator * ( const Transform &t ) const ;
  
        // applies 't' to the receiver, and returns
        // a copy of the result      
  
  friend Point3D  LinearInterpolate( Scalar t, const Point3D &p0, const Point3D &p1 ) ;

} ;



#endif  // VECTORH_HPP
